import Button from '@/components/elements/Button';
import Spinner from '@/components/elements/Spinner';
import React, { ChangeEvent, useEffect, useRef, useState } from 'react';
import FyreHostConfig from '@/components/FyreHostConfig';

import tw from 'twin.macro';
import ModContainer from './ModContainer';
import http from '@/api/http';
import { ServerContext } from '@/state/server';
import Input from '@/components/elements/Input';
import searchMods from '@/api/server/mods/searchMods';
import getMod from '@/api/server/mods/getMod';
import { Mod } from './types';
import loadDirectory from '@/api/server/files/loadDirectory';

export default () => {
    const uuid = ServerContext.useStoreState((state) => state.server.data!.uuid);

    const temp: JSX.Element[] = [];

    const temp2: Mod[] = [];

    const [mods, setMods] = useState(temp2);

    const [renderedMods, setRenderedMods] = useState(temp);

    const [search, setSearch] = useState('');

    const [page, setPage] = useState(0);

    const [loading, setLoading] = useState(true);

    const [category, setCategory] = useState(6);

    const [version, setVersion] = useState('');

    const inputRef = useRef(null);

    async function getDefaultPage(): Promise<Mod[]> {
        setLoading(true);

        if (category === 10) {
            const mods = await loadDirectory(uuid, 'mods/');

            const reqs: Promise<any>[] = [];

            for (let i = 0; i < mods.length; i++) {
                if (mods[i].name.split('-').length >= 3) {
                    const modId = parseInt(mods[i].name.split('-')[mods[i].name.split('-').length - 2].substring(1));

                    reqs.push(getMod(modId));
                }
            }

            const data = await Promise.all(reqs);

            const rd: Mod[] = [];

            data.forEach((e) => {
                rd.push(e);
            });

            return rd;
        }

        return await searchMods(page, search, category);
    }

    async function setModsDefault() {
        const oldSearch = search.slice();

        const p = await getDefaultPage();

        if (oldSearch !== inputRef.current!['value']) {
            return;
        }

        setMods(p);
    }

    function render() {
        if (mods) {
            const temp: JSX.Element[] = [];

            for (let i = 0; i < mods.length; i++) {
                temp.push(<ModContainer key={Math.random()} gameVersion={version} mod={mods[i]} />);
            }

            setRenderedMods(temp);
        }

        setLoading(false);
    }

    function onSearch(event: ChangeEvent<HTMLInputElement>) {
        setSearch(event.target.value);
    }

    function increasePage() {
        setPage(page + 1);
    }

    function decreasePage() {
        setPage(page - 1);
    }

    useEffect(() => {
        setLoading(true);
    }, []);

    useEffect(() => {
        render();
    }, [mods, version]);

    useEffect(() => {
        setModsDefault();
    }, [search, page, category]);

    return (
        <>
            <div css={tw`mx-10 mt-5 sm:flex`}>
                <div
                    css={tw`flex w-full sm:w-auto my-2 sm:my-0 rounded-full border-2`}
                    style={{
                        backgroundColor: FyreHostConfig.style.primaryColor,
                        borderColor: FyreHostConfig.style.secondaryColor,
                    }}
                >
                    <button
                        style={{
                            backgroundColor: FyreHostConfig.style.primaryColor,
                            borderColor: FyreHostConfig.style.secondaryColor,
                        }}
                        css={tw`w-full sm:w-8 rounded-l-full hover:font-black`}
                        disabled={page === 0}
                        onClick={decreasePage}
                    >
                        &#60;
                    </button>
                    <div
                        style={{
                            backgroundColor: FyreHostConfig.style.primaryColor,
                            borderColor: FyreHostConfig.style.secondaryColor,
                        }}
                        css={tw`text-2xl pt-2 border-0 h-12 w-full sm:w-8 text-center`}
                    >
                        {page + 1}
                    </div>
                    <button
                        style={{
                            backgroundColor: FyreHostConfig.style.primaryColor,
                            borderColor: FyreHostConfig.style.secondaryColor,
                        }}
                        css={tw`w-full sm:w-8 rounded-r-full hover:font-black`}
                        disabled={renderedMods.length < FyreHostConfig.pluginInstaller.amountPerPage}
                        onClick={increasePage}
                    >
                        &#62;
                    </button>
                </div>
                <input
                    onChange={onSearch}
                    ref={inputRef}
                    size={20}
                    css={tw`rounded-full bg-neutral-700 border-2 border-cyan-700 text-lg p-2 sm:ml-5 my-4 sm:my-0 w-full`}
                    style={{ backgroundColor: '#202020', borderColor: '#101010' }}
                    placeholder={'Search'}
                />
                <select
                    style={{
                        backgroundColor: FyreHostConfig.style.primaryColor,
                        borderColor: FyreHostConfig.style.secondaryColor,
                    }}
                    css={tw`rounded-full bg-neutral-700 border-2 text-lg p-2 sm:ml-5 my-2 sm:my-0 w-full sm:w-auto text-center`}
                    value={version}
                    onChange={(event) => {
                        setVersion(event.target.value);
                    }}
                >
                    <option value={'Any'}>Any</option>
                    <option value={'1.7.10'}>1.7.10</option>
                    <option value={'1.8.9'}>1.8.9</option>
                    <option value={'1.9.4'}>1.9.4</option>
                    <option value={'1.10.2'}>1.10.2</option>
                    <option value={'1.11.2'}>1.11.2</option>
                    <option value={'1.12.2'}>1.12.2</option>
                    <option value={'1.13.2'}>1.13.2</option>
                    <option value={'1.14.4'}>1.14.4</option>
                    <option value={'1.15.2'}>1.15.2</option>
                    <option value={'1.16.5'}>1.16.5</option>
                    <option value={'1.17.1'}>1.17.1</option>
                    <option value={'1.18.2'}>1.18.2</option>
                    <option value={'1.19.2'}>1.19.2</option>
                </select>
                <select
                    style={{ backgroundColor: '#202020', borderColor: '#101010' }}
                    css={tw`rounded-full bg-neutral-700 border-2 border-cyan-700 text-lg p-2 sm:ml-5 w-full sm:w-auto`}
                    defaultValue={category}
                    onChange={(event) => {
                        setCategory(parseInt(event.target.value));
                    }}
                >
                    <option value={0}>none</option>
                    <option value={1}>Featured</option>
                    <option value={2}>Popularity</option>
                    <option value={3}>Last Updated</option>
                    <option value={4}>Name</option>
                    <option value={5}>Author</option>
                    <option value={6}>Total Downloads</option>
                    <option value={7}>Category</option>
                    <option value={8}>Game Version</option>
                    <option value={10}>Installed</option>
                </select>
            </div>
            {loading ? (
                <Spinner size='large' centered></Spinner>
            ) : (
                <>
                    <div css={tw`grid md:grid-rows-4 md:grid-cols-4 mt-4 mx-10 gap-4`}>{renderedMods}</div>
                </>
            )}
        </>
    );
};
